@@ -152,21 +152,21 @@ metal_charge_dict = {
     'La': 3, 'Ce': 3, 'Pr': 3, 'Nd': 3, 'Pm': 3, 'Sm': 3, 'Eu': 3, 'Gd': 3, 'Tb': 3, 'Dy': 3,
     'Ho': 3, 'Er': 3, 'Tm': 3, 'Yb': 3, 'Lu': 3,
     # Actinides
-    'Ac': 3, 'Th': 4, 'Pa': 5, 'U': 4, 'Np': 5, 'Pu': 4, 'Am': 3, 'Cm': 3, 'Bk': 3, 'Cf': 3,
+    'Ac': 3, 'Th': 4, 'Pa': 5, 'U': 4, 'Np': 4, 'Pu': 4, 'Am': 3, 'Cm': 3, 'Bk': 3, 'Cf': 3,
     'Es': 3, 'Fm': 3, 'Md': 3, 'No': 2, 'Lr': 3,
-    # First row transition metals -> Switching to Fe 3 to encourge more convergence.
-    'Sc': 3, 'Ti': 2, 'V': 2,  'Cr': 2, 'Mn': 2, 'Fe': 3, 'Co': 2, 'Ni': 2, 'Cu': 1, 'Zn': 2, 
+    # First row transition metals 
+    'Sc': 3, 'Ti': 4, 'V': 5,  'Cr': 3, 'Mn': 2, 'Fe': 2, 'Co': 2, 'Ni': 2, 'Cu': 2, 'Zn': 2, 
     # Second row transition metals 
-    'Y': 3, 'Zr': 4, 'Nb': 5,  'Mo': 4, 'Tc': 4, 'Ru': 3, 'Rh': 3, 'Pd': 0, 'Ag': 1, 'Cd': 2, 
+    'Y': 3, 'Zr': 4, 'Nb': 5,  'Mo': 6, 'Tc': 5, 'Ru': 2, 'Rh': 1, 'Pd': 2, 'Ag': 1, 'Cd': 2, 
     # Third row transition metals
-    'Hf': 4, 'Ta': 5, 'W': 4,  'Re': 4, 'Os': 4, 'Ir': 3, 'Pt': 2, 'Au': 1, 'Hg': 1,
+    'Hf': 4, 'Ta': 5, 'W': 6,  'Re': 4, 'Os': 2, 'Ir': 3, 'Pt': 2, 'Au': 3, 'Hg': 2,
     # 4th row transition metals
     'Rf': 3, 'Db': 5, 'Sg': 6, 'Bh': 7, 'Hs': 8,
     # Post-transition metals
     'Al': 3,
     'Ga': 3,
-    'In': 3, 'Sn': 2,
-    'Tl': 1, 'Pb': 2, 'Bi': 3, 
+    'In': 3, 'Sn': 4,
+    'Tl': 3, 'Pb': 2, 'Bi': 3, 
     'Nh': 1, 'Fl': 2, 'Mc': 1, 'Lv': 2,
     # Alkali/earth metals
     'Li': 1, 'Be': 2 , 
@@ -183,14 +183,14 @@ metal_spin_dict = {
     'La': 0, 'Ce': 1, 'Pr': 2, 'Nd': 3, 'Pm': 4, 'Sm': 5, 'Eu': 6, 'Gd': 7, 'Tb': 6, 'Dy': 5,
     'Ho': 4, 'Er': 3, 'Tm': 2, 'Yb': 1, 'Lu': 0,
     # Actinides
-    'Ac': 0, 'Th': 0, 'Pa': 0, 'U': 2, 'Np': 2, 'Pu': 4, 'Am': 6, 'Cm': 7, 'Bk': 6, 'Cf': 5,
+    'Ac': 0, 'Th': 0, 'Pa': 0, 'U': 2, 'Np': 3, 'Pu': 4, 'Am': 6, 'Cm': 7, 'Bk': 6, 'Cf': 5,
     'Es': 4, 'Fm': 3, 'Md': 2, 'No': 0, 'Lr': 0,
     # First row transition metals
-    'Sc': 0, 'Ti': 2, 'V': 3,  'Cr': 4, 'Mn': 5, 'Fe': 4, 'Co': 3, 'Ni': 2, 'Cu': 0, 'Zn': 0, 
+    'Sc': 0, 'Ti': 0, 'V': 0,  'Cr': 3, 'Mn': 5, 'Fe': 4, 'Co': 3, 'Ni': 2, 'Cu': 1, 'Zn': 0, 
     # Second row transition metals 
-    'Y': 0, 'Zr': 0, 'Nb': 0,  'Mo': 2, 'Tc': 3, 'Ru': 5, 'Rh': 4, 'Pd': 0, 'Ag': 0, 'Cd': 0, 
+    'Y': 0, 'Zr': 0, 'Nb': 0,  'Mo': 0, 'Tc': 2, 'Ru': 4, 'Rh': 2, 'Pd': 0, 'Ag': 0, 'Cd': 0, 
     # Third row transition metals
-    'Hf': 0, 'Ta': 0, 'W': 2,  'Re': 3, 'Os': 4, 'Ir': 4, 'Pt': 2, 'Au': 0, 'Hg': 1,
+    'Hf': 0, 'Ta': 0, 'W': 0,  'Re': 1, 'Os': 0, 'Ir': 0, 'Pt': 0, 'Au': 2, 'Hg': 0,
     # 4th row transition metals
     'Rf': 1, 'Db': 0, 'Sg': 0, 'Bh': 0, 'Hs': 0,
     # Post-transition metals
@@ -247,6 +247,36 @@ second_choice_metal_spin_dict = {
     # 'Fr': 0, 'Ra': 0
     }
 
+metal_CN_dict = {
+    # Lanthanides
+    'La': [7,8,9], 'Ce': [7,8,9], 'Pr': [7,8,9], 'Nd': [7,8,9], 'Pm': [7,8,9], 'Sm': [7,8,9], 'Eu': [7,8,9], 'Gd': [7,8,9], 'Tb': [7,8,9], 'Dy': [7,8,9],
+    'Ho': [7,8,9], 'Er': [7,8,9], 'Tm': [7,8,9], 'Yb': [7,8,9], 'Lu': [7,8,9],
+    # Actinides
+    'Ac': [6,7,8,9], 'Th': [6,7,8,9], 'Pa': [6,7,8,9], 'U': [6,7,8,9], 'Np': [6,7,8,9], 'Pu': [6,7,8,9], 'Am': [6,7,8,9], 'Cm': [6,7,8,9], 'Bk': [6,7,8,9], 'Cf': [6,7,8,9],
+    'Es': [6,7,8,9], 'Fm': [6,7,8,9], 'Md': [6,7,8,9], 'No': [6,7,8,9], 'Lr': [6,7,8,9],
+    # First row transition metals 
+    'Sc': [4,6], 'Ti': [4,6], 'V': [4,6],  'Cr': [4,6], 'Mn': [4,6], 'Fe': [4,6], 'Co': [4,6], 'Ni': [4,6], 'Cu': [4,6], 'Zn': [4,6], 
+    # Second row transition metals 
+    'Y': [4,6], 'Zr': [4,6], 'Nb': [4,6],  'Mo': [4,6], 'Tc': [4,6], 'Ru': [4,6], 'Rh': [4,6], 'Pd': [4,6], 'Ag': [4,6], 'Cd': [4,6], 
+    # Third row transition metals
+    'Hf': [4,6], 'Ta': [4,6], 'W': [4,6],  'Re': [4,6], 'Os': [4,6], 'Ir': [4,6], 'Pt': [4,6], 'Au': [4,6], 'Hg': [4,6],
+    # 4th row transition metals
+    'Rf': [4,6], 'Db': [4,6], 'Sg': [4,6], 'Bh': [4,6], 'Hs': [4,6],
+    # Post-transition metals
+    'Al': [4,6],
+    'Ga': [4,6],
+    'In': [4,6], 'Sn': [4,6],
+    'Tl': [4,6], 'Pb': [4,6], 'Bi': [4,6], 
+    'Nh': [4,6], 'Fl': [4,6], 'Mc': [4,6], 'Lv': [4,6],
+    # Alkali/earth metals
+    'Li': [4,6,8], 'Be': [4,6,8], 
+    'Na': [4,6,8], 'Mg': [4,6,8], 
+    'K': [4,6,8], 'Ca': [4,6,8], 
+    'Rb': [4,6,8], 'Sr': [4,6,8], 
+    'Cs': [4,6,8], 'Ba': [4,6,8], 
+    'Fr': [4,6,8], 'Ra': [4,6,8]
+    }
+
 # GFN2-xTB single atom energies see io_xtb_calc.calc_xtb_ref_dict for reference.
 xtb_single_atom_ref_es = {'H': -10.707211383396714, 
  'He': -47.432891698445495,
@@ -421,7 +451,8 @@ ligands_dict = {
 'benzoate_mono':{'smiles':'C1=CC=C(C=C1)C(=O)[O-]','coordList':[8],'ligType':'mono'},
 'benzoic_acid':{'smiles':'C1=CC=C(C=C1)C(=O)O','coordList':[7],'ligType':'mono'},
 'oxalate':{'smiles':'C(=O)(C(=O)[O-])[O-]','coordList':[4,5],'ligType':'bi_cis'},
-'nitrite':{'smiles':'N(=O)[O-]','coordList':[2],'ligType':'mono'},
+'nitrite_o':{'smiles':'N(=O)[O-]','coordList':[2],'ligType':'mono'},
+'nitrite_n':{'smiles':'N(=O)[O-]','coordList':[0],'ligType':'mono'},
 'acetonitrile':{'smiles':'CC#N','coordList':[2],'ligType':'mono'},
 'ammonia':{'smiles':'N','coordList':[0],'ligType':'mono'},
 'thf':{'smiles':'C1CCCO1','coordList':[4],'ligType':'mono'},
@@ -459,6 +490,9 @@ ligands_dict = {
         'ligType':'octa_square_antiprismatic'},
 'dtpa':{'smiles':'C(CN(CC(=O)O)CC(=O)O)N(CCN(CC(=O)O)CC(=O)O)CC(=O)O', 
         'coordList':[2,5,9,11,14,17,21,25],'ligType':'octa_square_antiprismatic'},
+'3,4,3-li(1-2-hopo)':{'smiles':'C1=CC(=O)N(C(=C1)C(=O)NCCCN(CCCCN(CCCNC(=O)C2=CC=CC(=O)N2[O-])C(=O)C3=CC=CC(=O)N3[O-])C(=O)C4=CC=CC(=O)N4[O-])[O-]', 
+        'coordList':[3,53,52,50,30,32,40,42],'ligType':'octa_square_antiprismatic'},
+'dheba':{'smiles':'CCCCC(CC)COP(=O)([O-])OCC(CC)CCCC','coordList':[10,11],'ligType':'bi_cis_bulky'},
 'edta':{'smiles':'[O-]C(=O)CN(CCN(CC([O-])=O)CC([O-])=O)CC([O-])=O','coordList':[0,4,7,10,14,18],
         'ligType':'hexa_octahedral'},
 'ethylenediaminetriacetate':{'smiles':'C(C[NH+](CC(=O)[O-])CC(=O)[O-])NCC(=O)[O-]',
